/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao;

import java.lang.reflect.Method;
import java.util.List;

/**
 * Implemented by AutoDAO implementations. You'll only want to use this interface if you're writing your own
 * omplementation of AutoDAO.
 */
public interface FinderExecutor {

  /**
   * Execute a finder method with the appropriate arguments.
   * @param method    query method.
   * @param queryArgs arguments for query method
   * @param limit     query limit. Can be <code>null</code>. See {@link Limit}.
   * @param offset    query offset. Can be <code>null</code>. See {@link Offset}.
   * @return list of found objects or empty list.
   */
  <T> List<T> executeListFinder(final Method method, final Object[] queryArgs, final Integer limit,
                                final Integer offset);

  /**
   * Same as {@link #executeListFinder(java.lang.reflect.Method, Object[], Integer, Integer)}  but for single result.
   * @param method    query method.
   * @param queryArgs arguments for query method
   * @return found object or <code>null</code>
   */
  <T> T executeSingleFinder(Method method, Object[] queryArgs);
}
